@@ -1,9 +1,8 @@
 # ABSTRACT: Simpler REST client for Icinga2
 
 package Monitoring::Icinga2::Client::Simple;
-$Monitoring::Icinga2::Client::Simple::VERSION = '0.002000_01'; # TRIAL
-
-$Monitoring::Icinga2::Client::Simple::VERSION = '0.00200001';use strict;
+$Monitoring::Icinga2::Client::Simple::VERSION = '0.001000';
+use strict;
 use warnings;
 use 5.010_001;
 use Monitoring::Icinga2::Client::REST 2;
@@ -52,7 +51,7 @@ sub schedule_downtime {
 
 sub _schedule_downtime_type {
     my ($self, $type, $args) = @_;
-    my $req_results = $self->_request('POST',
+    my $req_results = $self->_verbose_request('POST',
         '/actions/schedule-downtime',
         {
             type => $type,
@@ -94,7 +93,7 @@ sub _remove_downtime_type {
     } else {
         @post_args = ( $args, { type => $type } );
     }
-    my $req_results = $self->_request('POST',
+    my $req_results = $self->_verbose_request('POST',
         "/actions/remove-downtime",
         @post_args,
     );
@@ -108,7 +107,7 @@ sub send_custom_notification {
 
     my $obj_type = defined $args{host} ? 'host' : 'service';
 
-    return $self->_request('POST',
+    return $self->_verbose_request('POST',
         '/actions/send-custom-notification',
         {
             type => ucfirst $obj_type,
@@ -127,7 +126,7 @@ sub set_notifications {
     _checkargs_any(\%args, qw/ host service /);
     my $uri_object = $args{service} ? 'services' : 'hosts';
 
-    return $self->_request('POST',
+    return $self->_verbose_request('POST',
         "/objects/$uri_object",
         {
             attrs => { enable_notifications => !!$args{state} },
@@ -139,7 +138,7 @@ sub set_notifications {
 sub query_app_attrs {
     my ($self) = @_;
 
-    my $r = $self->_request('GET',
+    my $r = $self->_verbose_request('GET',
         "/status/IcingaApplication",
     );
     # uncoverable branch true
@@ -163,14 +162,14 @@ sub query_app_attrs {
     sub set_app_attrs {
         my ($self, %args) = @_;
         _checkargs_any(\%args, keys %legal_attrs);
-        my @unknown_attrs = grep { not exists $legal_attrs{$_} } keys %args;
+        my @unknown_attrs = grep { not exists $legal_attrs{$_} } sort keys %args;
         @unknown_attrs and croak(
             sprintf "Unknown attributes: %s; legal attributes are: %s",
-            join(",", sort @unknown_attrs),
+            join(",", @unknown_attrs),
             join(",", sort keys %legal_attrs),
         );
 
-        return $self->_request('POST',
+        return $self->_verbose_request('POST',
             '/objects/icingaapplications/app',
             {
                 attrs => {
@@ -186,54 +185,34 @@ sub set_global_notifications {
     $self->set_app_attrs( notifications => $state );
 }
 
-sub query_hosts {
-    my ($self, %args) = @_;
-    _checkargs(\%args, qw/ hosts /);
-    my $result = $self->_request('GET',
-        '/objects/hosts',
-        { filter => _filter_expr( "host.name", $args{hosts} ) },
-    );
-}
-
 sub query_host {
     my ($self, %args) = @_;
     _checkargs(\%args, qw/ host /);
-    return $self->query_hosts( hosts => $args{host} )->[0];
+    return $self->_verbose_request('GET',
+        '/objects/hosts',
+        { filter => "host.name==\"$args{host}\"" }
+    )->[0];
 }
 
 sub query_child_hosts {
     my ($self, %args) = @_;
     _checkargs(\%args, qw/ host /);
-    return $self->_request('GET',
+    return $self->_verbose_request('GET',
         '/objects/hosts',
         { filter => "\"$args{host}\" in host.vars.parents" }
     );
 }
 
-sub query_parent_hosts {
-    my ($self, %args) = @_;
-    my $expand = delete $args{expand};
-    # uncoverable condition right
-    my $results = $self->query_host( %args ) // {};
-    # uncoverable condition right
-    my $names = $results->{attrs}{vars}{parents} // [];
-    undef $results;
-    # uncoverable condition right
-    return $names unless $expand and @$names;
-    return $self->query_hosts( hosts => $names );
-}
-
 sub query_services {
     my ($self, %args) = @_;
-    _checkargs_any(\%args, qw/ service services /);
-    my $srv = $args{service} // $args{services};
-    return $self->_request('GET',
+    _checkargs(\%args, qw/ service /);
+    return $self->_verbose_request('GET',
         '/objects/services',
-        { filter => _filter_expr( "service.name", $srv ) },
+        { filter => "service.name==\"$args{service}\"" }
     );
 }
 
-sub _request {
+sub _verbose_request {
     my ($self, $method, $url, $getargs, $postdata) = @_;
 
     if(defined $getargs and ref $getargs) {
@@ -241,9 +220,12 @@ sub _request {
         $postdata = $getargs;
         undef $getargs;
     }
+    _debug("Query URL: $url", $getargs ? "Query args: $getargs" : ());
+    _debug_dump($postdata);
     # uncoverable branch true
     my $r = $self->do_request($method, $url, $getargs, $postdata)
         or die $self->request_status_line . "\n";
+    _debug_dump($r);
     return $r->{results};
 }
 
@@ -274,20 +256,21 @@ sub _checkargs_any {
 # Not a method!
 sub _create_filter {
     my $args = shift;
-    defined $args->{host} or croak(
-        sprintf( "missing or undefined argument `host' to %s()", (caller(1))[3] )
-    );
-    my $filter = _filter_expr( "host.name", $args->{host} );
-    return $filter unless $args->{service};
-    return "$filter && " . _filter_expr( "service.name", $args->{service} );
+    croak( "`host' argument missing" ) unless defined $args->{host};
+    my $filter = "host.name==\"$args->{host}\"";
+    $filter .= " && service.name==\"$args->{service}\"" if $args->{service};
+    return $filter;
 }
 
-# Return an == or `in' expression depending on the type of argument.
-# Only scalars and arrayrefs make sense!
-sub _filter_expr {
-    my ($what, $arg) = @_;
-    return "$what==\"$arg\"" unless ref $arg;
-    return "$what in [" . join( ',', map { "\"$_\"" } @$arg ) . ']';
+sub _debug {
+    print STDERR @_ if DEBUG;
+}
+
+sub _debug_dump {
+    if(DEBUG) {
+        require YAML::XS;
+        print YAML::XS::Dump(\@_);
+    }
 }
 
 1;
@@ -304,7 +287,7 @@ Monitoring::Icinga2::Client::Simple - Simpler REST client for Icinga2
 
 =head1 VERSION
 
-version 0.002000_01
+version 0.001000
 
 =head1 SYNOPSIS
 
@@ -512,24 +495,12 @@ argument to also delete all of this host's service downtimes.
     $result = $ia->query_host( host => 'web-1' );
     say "$result->{attrs}{name}: $result->{attrs}{type}";
 
-Equivalent to L</query_hosts> except that it accepts only a single host name
-and the result is a reference to a single hash, or C<undef> if the host
-can't be found.
+Query all information Icinga2 has on a certain host. The result is a hashref,
+currently containing a single key C<attrs>. If the host is not found, C<undef>
+is returned.
 
 The only mandatory argument is C<host>.
 
-=head2 query_hosts
-
-    $results = $ia->query_host( hosts => [ qw( web-1 hypervisor-1 ) ] );
-    say "$_->{attrs}{name}: $_->{attrs}{type}" for @$results;
-
-Query all information Icinga2 has on a number of hosts. The result is a
-reference to a list of hashes, each of which currently contains a single key
-C<attrs>.
-
-The only mandatory argument is C<hosts>, which may also specify a single
-scalar, so this method is preferable for consistency over L</query_host>.
-
 =head2 query_child_hosts
 
     $results = $ia->query_child_hosts( host => 'hypervisor-1' );
@@ -540,28 +511,18 @@ is a reference to a list of hashes like those returned by L</query_host>.
 
 The only mandatory argument is C<host>.
 
-=head2 query_parent_hosts
-
-    $results = $ia->query_parent_hosts( host => 'web-1', expand => 1 );
-    say for @$results;
-
-Query all parent host names of a given host. If C<expand> is C<false> or
-missing, the result is a reference to a list of names. Otherwise it is a
-reference to a list of hashes like those returned by L</query_host>.
-
-The only mandatory argument is C<host>.
-
 =head2 query_services
 
     $result = $ia->query_services( service => 'HTTP' );
-    $result = $ia->query_services( services => [ qw( HTTP SMTP ) ] );
     say "$_->{attrs}{name}: $_->{attrs}{type}" for @$results;
 
-Query all information Icinga2 has on a number of services. As services usually
-have more than one instance, the result is always a reference to a list of hashes,
-each describing one instance, even when querying a single service.
+Query all information Icinga2 has on a certain service. As services usually
+have more than one instance, the result is a reference to a list of hashes,
+each describing one instance.
 
-The only mandatory argument is C<service> I<or> C<services>.
+The only mandatory argument is C<service>. Note that this is a singular as it
+specifies a single service name while the method name is plural due to the
+plurality of returned results.
 
 =head2 send_custom_notification
 
